#ifndef __RPCSVC_YP_H__
#include <nis/rpcsvc/yp.h>

libnsl_hidden_proto (xdr_ypstat)
libnsl_hidden_proto (xdr_ypxfrstat)
libnsl_hidden_proto (xdr_domainname)
libnsl_hidden_proto (xdr_mapname)
libnsl_hidden_proto (xdr_peername)
libnsl_hidden_proto (xdr_keydat)
libnsl_hidden_proto (xdr_valdat)
libnsl_hidden_proto (xdr_ypmap_parms)
libnsl_hidden_proto (xdr_ypresp_key_val)
libnsl_hidden_proto (xdr_ypresp_all)
libnsl_hidden_proto (xdr_yppush_status)
libnsl_hidden_proto (xdr_ypbind_resptype)
libnsl_hidden_proto (xdr_ypbind_binding)

#endif
